/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications for any purpose,
 * commercial or otherwise. 
 * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
 *********************************/

package fanchart;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Point2D;

public class ForceAncestorsWedge {
	private HyrumPolygon mainPolygon;
	private HyrumPolygon triangle;
	private float start, length;

	public ForceAncestorsWedge(float start, float length) {
		mainPolygon = new HyrumPolygon();
		triangle = new HyrumPolygon();
		this.start = start;
		this.length = length;
	}

	public void buildVertices(RadialCache cache, int rad, int offset) {
		mainPolygon.reset();
		triangle.reset();
		int res = RadialCache.RESOLUTION;

		int index0 = Math.round(offset + start);
		if (index0 < 0) index0 += res;
		int llength = Math.round(length);

		for (int i=0; i<=llength; ++i) {	
			mainPolygon.addPoint(cache.getVertex(rad, (index0 + i) % res));
		}
		double hypotenuse = Config.baseWedgeThickness;
		double longerHypotenuse = hypotenuse + Config.expansionWedgeThickness;
		double ratio = 1.0 - (hypotenuse / longerHypotenuse);
		Point2D.Float tmp1, tmp2, tmp3;
		for (int i=llength; i>=0; --i) {
			tmp1 = cache.getVertex(rad, (index0 + i) % res);
			tmp2 = cache.getVertex(rad-1, (index0 + i) % res);
			tmp3 = new Point2D.Float((float)(tmp1.x + (ratio * (tmp1.x - tmp2.x))),
					 (float)(tmp1.y + (ratio * (tmp1.y - tmp2.y))));
			mainPolygon.addPoint(tmp3.x, tmp3.y);
		}		
		mainPolygon.closePath();
		
		//create the triangle
		double spread = Math.toRadians(25);
		double len = Config.expansionWedgeThickness/2;
		ratio = 0.8;
		tmp1 = mainPolygon.getPointAt((int)(length/2f));
		tmp2 = mainPolygon.getPointAt((int)(length*1.5f));
		tmp3 = new Point2D.Float((float)(tmp1.x + (ratio * (tmp2.x - tmp1.x))),
				 (float)(tmp1.y + (ratio * (tmp2.y - tmp1.y))));
		triangle.addPoint(tmp3.x, tmp3.y);
		double lineAngle = Math.atan2((double)(tmp3.y - tmp1.y), (double)(tmp1.x - tmp3.x));
		double angle1 = lineAngle - spread;
		double angle2 = lineAngle + spread;
		tmp2 = new Point2D.Float(tmp3.x + (float)(len*Math.cos(angle1)), tmp3.y - (float)(len*Math.sin(angle1)));
		triangle.addPoint(tmp2.x, tmp2.y);
		tmp2 = new Point2D.Float(tmp3.x + (float)(len*Math.cos(angle2)), tmp3.y - (float)(len*Math.sin(angle2)));
		triangle.addPoint(tmp2.x, tmp2.y);

		triangle.closePath();
	}
	
	public void draw(Graphics2D g) {
		g.setColor(Config.wedgeBorderColor);
		g.setStroke(Config.regularLine);
		mainPolygon.draw(g);
		triangle.fill(g);
	}
	
	public boolean containsPoint(Point p) {
		return mainPolygon.contains(p);
	}
}
