/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications for any purpose,
 * commercial or otherwise. 
 * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
 *********************************/

package fanchart;



import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JApplet;
import javax.swing.JOptionPane;
import javax.swing.UIManager;


public class MainApplet extends JApplet implements ActionListener {
	private static MainApplet instance = null;
	private int windowWidth, windowHeight;
	private static Ward ward;
	private FanChartWidget mainPanel;
	private static final long serialVersionUID = 42L;
	private boolean testing = true;

	public static MainApplet instance() {
		if(instance == null) {
			instance = new MainApplet();
		}
		return instance;
	}

	public MainApplet() {
		super();

		Config.init();

		//this makes the GUI adopt the look-n-feel of the windowing system (Windows/X/Mac)
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) { }


		windowWidth = Config.DEFAULT_WINDOW_WIDTH;
		windowHeight = Config.DEFAULT_WINDOW_HEIGHT;
		Dimension dim = new Dimension(windowWidth, windowHeight);
		setPreferredSize(dim);

		//enable smooth window resizing
		//TODO this may not be necessary in JDK1.6
		Toolkit.getDefaultToolkit().setDynamicLayout(true);

		ward = new Ward();

		if (testing) {
			ward.testing();
		} else {
			//TODO load data from database
		}
		ward.configureGenerations();
		mainPanel = new FanChartWidget(ward);
		
		setJMenuBar(new MyMenuBar(this));
		getJMenuBar().remove(0);//HACK: get rid of the File/Exit menu in the applet version

		setContentPane(mainPanel);
		setVisible(true);
	}

	/**
	 * Handles all the menu actions.
	 * In reality, this just defers to my homebrew
	 * "handleAction" method.
	 */
	public void actionPerformed(ActionEvent e) {
		String name = e.getActionCommand();
		handleAction(name);
	}

	private void handleAction(String name) {
		if (name.equals(MyMenuBar.EXIT_MENU_OPTION)) {
			System.exit(0);
		}
		if (name.equals(MyMenuBar.ABOUT_MENU_OPTION)) {
			JOptionPane.showMessageDialog(this,
					"<html>Interactive Fan Chart Demo<br>&nbsp;<br>(C) 2008 Geoffrey M. Draper</html>",
					"About",
					JOptionPane.PLAIN_MESSAGE);
		}
		if (name.equals(MyMenuBar.SET_NUM_GENERATIONS_OPTION)) {
			new NumGenerationsDialog(this, ward);
		}
		if (name.equals(MyMenuBar.TOGGLE_ANIMATION_OPTION)) {
			Config.db_animate = !Config.db_animate;
		}
		if (name.equals(MyMenuBar.TOGGLE_CLICK_TO_EXPAND)) {
			Config.clickToExpand = !Config.clickToExpand;
		}
		if (name.equals(MyMenuBar.SELECT_ARBITRARY_ROOT)) {
			new PersonSelectorDialog(this, ward);
		}
	}
}
