/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications for any purpose,
 * commercial or otherwise. 
 * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
 *********************************/

package fanchart;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BoxLayout;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

public class NumGenerationsDialog extends JDialog implements ActionListener {
	
	private JSpinner numberWidget;
	private Ward ward;
	private static final long serialVersionUID = 42L;
	private static final String OK_BUTTON_LABEL = "OK";
	private static final String CANCEL_BUTTON_LABEL = "Cancel";
	
	public NumGenerationsDialog(Frame parent, Ward ward) {
		super(parent);
		init(parent, ward);
	}
	
	public NumGenerationsDialog(JApplet parent, Ward ward) {
		super();
		init(parent, ward);
	}
	
	private void init(Component parent, Ward ward) {
		this.ward = ward;
		setTitle("How many generations");
		buildGUI();
		setResizable(false);
		pack();
		
		//position this dialog in the middle of its parent window
		Rectangle parentBounds = parent.getBounds();
		Dimension mySize = getSize();
		setLocation(parentBounds.x + parentBounds.width/2 - mySize.width/2,
				parentBounds.y + parentBounds.height/2 - mySize.height/2);
		
		
		//treat a window-closing event as if the user
		//had clicked the "cancel" button.
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent event) {
				handleActionEvent(CANCEL_BUTTON_LABEL);
			}
		});
		
		setVisible(true);
		
	}
	
	private void buildGUI() {
		setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS));
		JLabel percentLabel = new JLabel("Please enter the number of generations to show:");
		percentLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		SpinnerModel spinnerModel =
			new SpinnerNumberModel(
					Config.numberOfGenerationsToShow, //initial value
					2,		//min
					100,		//max
					1);		//step
		numberWidget = new JSpinner(spinnerModel);
		
		JPanel okCancelPanel = new JPanel();
		okCancelPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		JButton okButton = new JButton(OK_BUTTON_LABEL);
		getRootPane().setDefaultButton(okButton);
		JButton cancelButton = new JButton(CANCEL_BUTTON_LABEL);
		okCancelPanel.add(okButton);
		okCancelPanel.add(cancelButton);
		
		okButton.setActionCommand(OK_BUTTON_LABEL);
		cancelButton.setActionCommand(CANCEL_BUTTON_LABEL);
		okButton.addActionListener(this);
		cancelButton.addActionListener(this);
		
		add(new JPanel());//space filler
		add(percentLabel);
		add(new JPanel());//space filler
		add(numberWidget);
		add(new JPanel());//space filler
		add(okCancelPanel);
	}
	
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		handleActionEvent(command);
	}
	
	private void handleActionEvent(String command) {
		if (command.equals(OK_BUTTON_LABEL)) {
			int val = (Integer)numberWidget.getValue();
			if (val > 1) {
				Config.setNumGenerations(val);
				ward.assignmentsChanged();
			}
		}
		setVisible(false);
	}
	
}
